@@ -1,8 +1,8 @@
-/*  $Id: BoolExpr.cpp,v 1.22 2008/10/09 03:14:05 sarrazip Exp $
+/*  $Id: BoolExpr.cpp,v 1.24 2010/02/14 19:28:30 sarrazip Exp $
     BoolExpr.cpp - Boolean expression binary tree node
 
     boolstuff - Disjunctive Normal Form boolean expression library
-    Copyright (C) 2002-2005 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2002-2010 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -25,6 +25,8 @@
 #endif
 
 #include <set>
+#include <algorithm>
+#include <stdlib.h>
 
 
 // Define this to enable traces that help in leak detection
@@ -370,6 +372,61 @@ BoolExpr<T>::cloneTree(const BoolExpr<T> *root)
 }
 
 
+// Returns -1 if term a comes before term b, +1 if term a comes after term b,
+// or 0 if they are equal.
+//
+template <class T>
+static int exprComparator(BoolExpr<T> *a, BoolExpr<T> *b)
+{
+    if (a == NULL && b != NULL)
+	return -1;
+    if (b == NULL)
+	return a == NULL ? 0 : +1;
+
+    typename BoolExpr<T>::Type aType = a->getType();
+    typename BoolExpr<T>::Type bType = b->getType();
+    if (aType != bType)
+	return int(aType) < int(bType) ? -1 : +1;
+
+    switch (aType)
+    {
+	case BoolExpr<T>::VALUE:
+	{
+	    const T &aValue = a->getValue();
+	    const T &bValue = b->getValue();
+	    if (aValue < bValue)
+		return -1;
+	    if (bValue < aValue)
+		return +1;
+	    return 0;
+	}
+	case BoolExpr<T>::AND:
+	case BoolExpr<T>::OR:
+	{
+	    int leftComparison = exprComparator(a->getLeft(),  b->getLeft());
+	    if (leftComparison != 0)
+		return leftComparison;
+	}
+
+	/*** NO BREAK ***/
+
+	case BoolExpr<T>::NOT:
+	    return exprComparator(a->getRight(), b->getRight());
+    }
+
+    return true;
+}
+
+
+// Returns true iff term a comes before term b.
+//
+template <class T>
+static bool termOrdering(BoolExpr<T> *a, BoolExpr<T> *b)
+{
+    return exprComparator(a, b) < 0;
+}
+
+
 template <class T>
 /*static*/
 BoolExpr<T> *
@@ -392,14 +449,26 @@ BoolExpr<T>::getDisjunctiveNormalForm(BoolExpr<T> *root)
 	return termRoots.front();
     }
 
-    // Determine which terms are useful:
+    // Sort terms to find repetitions:
+    std::sort(termRoots.begin(), termRoots.end(), termOrdering<T>);
+
+    // Determine which terms are useful (terms that don't always evaluate to false,
+    // and are not the same as a previous term):
     std::vector<BoolExpr<T> *> usefulTerms;
     std::vector<bool> isUsefulTerm;
     typename std::vector<BoolExpr<T> *>::const_iterator it;
-    for (it = termRoots.begin(); it != termRoots.end(); it++)
+    typename std::vector<BoolExpr<T> *>::const_iterator previous = termRoots.end();
+    for (it = termRoots.begin(); it != termRoots.end(); previous = it, it++)
     {
 	assert(*it != NULL);
 	bool u = (*it)->isDNFTermUseful();
+
+	// If 'it' is not the first term in termRoots, and 'it' is equal to
+	// the previous term in termRoots, then 'it' is not useful.
+	//
+	if (previous != termRoots.end() && exprComparator(*previous, *it) == 0)
+	    u = false;
+
 	isUsefulTerm.push_back(u);
 	if (u)
 	    usefulTerms.push_back(const_cast<BoolExpr<T> *>(*it));
