@@ -1,8 +1,8 @@
-/*  $Id: c-api.cpp,v 1.17 2008/10/07 02:29:03 sarrazip Exp $
+/*  $Id: c-api.cpp,v 1.19 2010/02/14 19:32:11 sarrazip Exp $
     c-api.cpp - C API function definitions
 
     boolstuff - Disjunctive Normal Form boolean expression library
-    Copyright (C) 2002-2005 Pierre Sarrazin <http://sarrazip.com/>
+    Copyright (C) 2002-2008 Pierre Sarrazin <http://sarrazip.com/>
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -244,7 +244,7 @@ void
 boolstuff_print_tree(FILE *out, boolexpr_t root)
 {
     string s = uncookie(root)->print();
-    fprintf(out, s.c_str());
+    fprintf(out, "%s", s.c_str());
 }
 
 
